function F = getOptimalFeatures(x,mz)
% Obtains features from inputs x and mz.
% x is the spectrum and has two columns -> 
% first column are m/z locations, 
% second column are the abundances.
% mz is the m/z of the precursor
% output is a vector F

%% general features
F1 = length(x(:,1)); % number of peaks

F5 = std(x(:,2)); % standard deviation

F8 = prctile(x(:,2),50); % 50th percentile (median) peak height

F9 = prctile(x(:,2),75);

F10 = prctile(x(:,2),95);

F23 = mz;

%% features from Klammer et al
addMass = [0 28 18 17]; % additional masses that could be added
k = 5; % parameters as discussed in paper
l = 5;

% generating S, where S(i) is the sum of all abundances within +/-0.5 of i m/z
z = round(x(:,1)); % rounding m/z locations to get bin indices
mz = round(mz); % rounding precursor mass to charge
S = zeros(mz*k,1);
for i = 1:length(z)
    S(z(i)) = S(z(i)) + x(i,2);
end

% these features capture various types of symmetries, 
% as described in Klammer et al (see that paper for more details)
F33 = 0;
j = 3;
for i = 1:mz*2-1-addMass(j)
    F33 = F33 + S(i)*S(2*mz-i-addMass(j));
end

F34 = 0;
j = 3;
for i = 1:(mz*3-1-addMass(j))/2
    F34 = F34 + S(i)*S(round(3*mz/2-i/2-addMass(j)/2));
end

F35 = F34/(F33 + .01); if(isnan(F35)); F35 = 1; end;

F51 = 0;
j = 1;
for i = 1:(5*mz-1-addMass(j))/4
    F51 = F51 + S(i)*S(round((5*mz-i-addMass(j))/4));
end

F58 = (sum(S(1:1*mz-1)) - sum(S(1*mz+1:4*mz)))./sum(S);

%% features from distributions of Klammer page 5
startRange = round((0:k*l-1)*mz/l + 1);
endRange = round((1:k*l)*mz/l);
f = zeros(1,l*k);
for i = 1:l*k
    f(i) = sum(S(startRange(i):endRange(i)));
end
f = f./sum(S);
F65 = f(3);
F68 = f(6);
F70 = f(8);
F71 = f(9);
F73 = f(11);

%% features from pairwise distance histograms
numBins = 2048; 
totalBins = log2(numBins); % uneven bin widths
endRange = 2.^(1:totalBins);

startRange = [1 endRange(1:end-1)+1];

% getting all pairwise differences and sums
L = length(x(:,1));
rep = repmat(x(:,1)',[L 1]);
% diffs of m/z
D1 = rep - rep';
keep = (D1 > 0); % need to keep only half due to anti-symmetry
D1 = D1(keep);

% generating a histogram for D
rep = repmat(x(:,2)',[L 1]);
W = min(rep,rep'); % weight = min of the two intensities (from Bern 04)
W = W(keep);

D1 = round(D1); % histograms will be on the integers
h1 = zeros(numBins,1); % weighted histogram
h2 = zeros(numBins,1); % unweighted histogram
for i = 1:length(D1) % generating histograms
    h1(D1(i)) = h1(D1(i)) + W(i);
    h2(D1(i)) = h2(D1(i)) + 1;
end

h1 = h1./sum(h1); % normalizing
h2 = h2./sum(h2); % normalizing

h1Binned = zeros(1,totalBins);
h2Binned = zeros(1,totalBins);
for i = 1:totalBins
    h1Binned(i) = sum(h1(startRange(i):endRange(i))); % summing over predefined ranges
    h2Binned(i) = sum(h2(startRange(i):endRange(i)));
end
h1 = h1Binned;
h2 = h2Binned;

F98 = h1(end);
F99 = h2(1);
F101 = h2(3);
F106 = h2(8);

F = [F1 F5 F8 F9 F10 F23 F35 F51 F58 F65 F68 F70 F71 F73 F98 F99 F101 F106];